<?php
/**
 * Advanced Ordernumbers generic helper class (e-commerce system agnostic)
 * Reinhold Kainhofer, Open Tools, office@open-tools.net
 * @copyright (C) 2012-2015 - Reinhold Kainhofer
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
**/

defined('_JEXEC') or	 die( 'Direct Access to ' . basename( __FILE__ ) . ' is not allowed.' ) ;

if (!class_exists( 'VmConfig' )) 
    require(JPATH_ROOT.DS.'administrator'.DS.'components'.DS.'com_virtuemart'.DS.'helpers'.DS.'config.php');
VmConfig::loadConfig();

if (!class_exists( 'OrdernumberHelper' )) 
	require_once (dirname(__FILE__) . DS . 'library' . DS . 'ordernumber_helper.php');

class OrdernumberHelperJoomla extends OrdernumberHelper {
	public $tableName = '';
	
	function __construct() {
		parent::__construct();
		// Joomla-specific Defaults for the HTML tables
		$this->_styles['counter-table-class'] = "table-striped";
		
		$this->ajax_url = "index.php?option=com_virtuemart&view=plugin&type=vmshopper&name=ordernumber&format=raw";
		
		// Joomla 2.x uses <li> for the params and float:left on the controls, so we need to add that too
		if (version_compare(JVERSION, '3.0', 'lt')) {
			$this->_styles['counter-table-style'] = "style=\"float: left;\"";
		}
 		
	}

	static function getHelper() {
		static $helper = null;
		if (!$helper) {
			$helper = new OrdernumberHelperJoomla();
		}
		return $helper;
    }
	
	// VM2 on J2 works, VM3 on J3 works out of the box, but
	// VM3 on J2 does NOT work by simply calling vmJsApi::jQuery, because
	// the JS is never added to the page header, so we have to add this manually
	public function loadjQuery() {
		vmJsApi::jQuery();
		// TODO: jquery::ui available only in J3:
		if (version_compare(JVERSION, '3.0', 'lt')) {
		} else {
			JHtml::_('jquery.ui', array('core', 'sortable'));
		}
		// If we are on Joomla 2.5 and VM 3, manually add the script declarations 
		// cached in vmJsApi to the document header:
		if (version_compare(JVERSION, '3.0', 'lt') && defined('VM_VERSION') && VM_VERSION>=3) {
			$document = JFactory::getDocument();
			$scripts = vmJsApi::getJScripts();
			foreach ($scripts as $name => $jsToAdd) {
				if($jsToAdd['written']) continue;
				$file = $jsToAdd['script'] ? $jsToAdd['script'] : $name;
				
				if(strpos($file,'/')!==0){
					$file = vmJsApi::setPath($file,false,'');
				} else if(strpos($file,'//')!==0){
					$file = JURI::root(true).$file;
				}

				$ver = '';
				if(!empty($jsToAdd['ver'])) $ver = '?vmver='.$jsToAdd['ver'];
				$document->addScript( $file .$ver,"text/javascript",$jsToAdd['defer'],$jsToAdd['async'] );
				vmJsApi::removeJScript($name);
			}
		}
	}
    

	
	public function __($string) {
// print("<pre>translating: $string</pre>");
		return JText::_($string);
	}
    function urlPath($type, $file) {
		static $pluginpath = '/plugins/vmshopper/ordernumber/library/';
		return JURI::root(true) . $pluginpath . $type . '/' . $file;
    }
 	
 	function getAllCounters($type) {
		// Look up the current counters
		$db = JFactory::getDBO();
		$db->setQuery('SELECT `number_type` AS `type`, `number_format` AS `name`, `count` AS `value` FROM `#__virtuemart_shopper_plg_ordernumber` WHERE `number_type`='.$db->quote($type) . ' ORDER BY `number_format`;' );
		return $db->loadObjectList();
 	}
 	
	function getCounter($nrtype, $format, $default=0) {
		$db = JFactory::getDBO();
        
		/* prevent sql injection attacks by escaping the user-entered format! Empty for global counter... */
		/* For global counting, simply read the empty number_format entries! */
		$q = 'SELECT `count` FROM `'.$this->tableName.'` WHERE `number_type`='.$db->quote($nrtype).' AND `number_format`='.$db->quote($format);
		$db->setQuery($q);
		$existing = $db->loadResult();
		$count = $existing?$existing:$default;
		return $count;
	}
    
	function counterExists($nrtype, $format) {
		$db = JFactory::getDBO();
		$q = 'SELECT `count` FROM `'.$this->tableName.'` WHERE `number_type`='.$db->quote($nrtype).' AND `number_format`='.$db->quote($format);
		$db->setQuery($q);
		return ($db->loadResult() != null);
	}
    
	// Insert new counter value into the db
	function addCounter($nrtype, $format, $value) {
		$db = JFactory::getDBO();
		$q = 'INSERT INTO `'.$this->tableName.'` (`count`, `number_type`, `number_format`) VALUES ('.(int)$value.','.$db->quote($nrtype).', '.$db->quote($format).')';
		$db->setQuery( $q );
		$db->query();
		return $db->getAffectedRows();
	}

	// Insert new counter value into the db or update existing one
	function setCounter($nrtype, $format, $value) {
		$db = JFactory::getDBO();
		$q = 'UPDATE `'.$this->tableName.'` SET `count`= "'.(int)$value.'" WHERE `number_type`='.$db->quote($nrtype).' AND `number_format`='.$db->quote($format);
		$db->setQuery( $q );
		$db->query();
		if ($db->getAffectedRows()<1) {
			return $this->addCounter($nrtype, $format, $value);
		} else {
			return $db->getAffectedRows();
		}
	}

	// Insert new counter value into the db or update existing one
	function deleteCounter($nrtype, $format) {
		$db = JFactory::getDBO();
		$format = $db->escape ($format);
		$q = 'DELETE FROM `'.$this->tableName.'` WHERE `number_type`='.$db->quote($nrtype).' AND `number_format`='.$db->quote($format);
		$db->setQuery( $q );
		$db->query();
		return $db->getAffectedRows();
	}

	function appendJS() {
		// Extract the messages from the returned string, add the ordernumber-message class (so the next ajax call
		// can remove them again) and then move the messages to the original message container.
		// Things are complicated by the fact that no #system-message element exists if no messages were printed so far
		// Also:
		// Request response as text, but interpret as JSON later in the done method (prevents a warning due to Joomla/VM's botched json implementation!)
		return '
ajax_ordernumber.updateMessages = function(messages, area) {
    jQuery( "#system-message-container #system-message ."+area+"-message").remove();
    var newmessages = jQuery( messages ).find("div.alert, .message").addClass(area+"-message");
    if (!jQuery( "#system-message-container #system-message").length && newmessages.length) {
        if (jQuery(newmessages).first().prop("tagName")=="dt") { // Joomla 2.x:
            jQuery( "#system-message-container" ).append( "<dl id=\'system-message\'></div>" );
        } else {
            jQuery( "#system-message-container" ).append( "<div id=\'system-message\'></div>" );
        }
    }
    newmessages.appendTo( "#system-message-container #system-message");
};
ajax_ordernumber.parseAjaxResponse = function(json) {
	return jQuery.parseJSON(json);
};
ajax_ordernumber.modifyAjaxArgs = function(ajaxargs) {
	ajaxargs.dataType="text";
	ajaxargs.cache=false;
	return ajaxargs;
};

';
	}


}
